// Formatted output stream
#ifndef _FIOstream_h
#define _FIOstream_h

#include <cstdio>
#include <string>

class FIOstream {
 public:
  enum Formats { Binary, Base64, Words };
  enum Mode { Read, Write };

  FIOstream (string name, Mode RW = Write)
    : isOpen_(false), moreInput_(false), nquartets_(0), nochars_(0)
    { open (name, RW); };
  ~FIOstream();
  bool open (string name, Mode RW = Write);
  // write a byte to the output
  void write (unsigned char ch);
  // Read a byte from the input.
  // Attempts to read past EOF will succeed, but return zero and set EOF.
  unsigned char read ();
  // eof() true means characters from read() are fake
  bool eof () const;
  // This is so we can do the weird test in SecretShare
  //bool weirdTest () const { return (!Stream_); };
  bool weirdTest () const { return eof(); };
  //fstream& stream () const { return Stream_; };
  static void setFormat (Formats t) { ioType_ = t; }

 private:
  FILE *F_;
  bool isOpen_;			// input file is open and not at EOF
  bool isWrite_;
  bool moreInput_;		// more input available
  // moreInput_ can be true even when isOpen_ is false, if there are 
  // still buffered characters available.

  static Formats ioType_;

  void close ();

  // Following for the Base64 encoder
  // This needs ochars_ to be only [3], but if we make it [8], the words
  // encoder/decoder can use it too.
  int nquartets_;
  unsigned char ochars_[8];
  int nochars_;
  static const char *base64Coding;
  static char *base64DeCoding;
  static const int charComment = 64;
  static const int charEOL = 65;
  static const int charIgnore = 66;

  void toBase64 (unsigned char ch);
  void flushBase64();
  unsigned char fromBase64 ();
  unsigned char getBase64Character();

  // Following for the english-words encoder/decoder
  char ebuf_[6*5+1];		// six * (word + space) plus termination
  void toWords (unsigned char ch);
  void flushWords ();
  unsigned char fromWords ();
};

#endif
