#ifndef _LagrangePolynomialInterpolation_h
#define _LagrangePolynomialInterpolation_h

#include "Point.h"
#include "Polynomial.h"

template <class NUMBER>
class LagrangePolynomialInterpolation {
public:

  LagrangePolynomialInterpolation() :
    points() {
  }

  virtual ~LagrangePolynomialInterpolation() {
  }

  void addPoint(const Point<NUMBER>& point) {
    points.push_back( point );
  }

  Polynomial<NUMBER> fit() const {
    Polynomial<NUMBER> result(0);

    for (unsigned j = 0; j < points.size(); j++) {
      result = result + Pj(j);
    }

    return result;
  }

  NUMBER fit(const NUMBER& x) const {
    NUMBER result(0);

    for (unsigned j = 0; j < points.size(); j++) {
      result += Pjx(j, x);
    }

    return result;
  }

  // Lazily go through the points, making sure that we don't have repeats.
  bool uniquePoints() const {
    for (unsigned i = 0; i < points.size(); i++) {
      for (unsigned j = 0; j < points.size(); j++) {
	if (i == j)
	  continue;
	
	if (points[i].x() == points[j].x()) {
	  cout << "Error: Two keys share identical points." << endl;
	  return false;
	}
      } 
    } 

    return true;
  }

protected:

  vector< Point<NUMBER> > points;

private:

  Polynomial<NUMBER> Pj(unsigned j) const {
    Polynomial<NUMBER> result(0);
    Polynomial<NUMBER> product(1);
 
    result.setCoefficient(0, points[j].y());

    for (unsigned i = 0; i < points.size(); i++) {
      if (i == j)
	continue;

      product.setCoefficient(1, (NUMBER) 1 / (points[j].x() - points[i].x()));
      product.setCoefficient(0, points[i].x() /
			     (points[i].x() - points[j].x()));
      
      // cout << "Sub-product (" << i << ", " << j << ") " << product << endl;
      
      result = result * product;
    }

    return result;
  }

  NUMBER Pjx(unsigned j, const NUMBER& x) const {
    NUMBER result(points[j].y());

    for (unsigned i = 0; i < points.size(); i++) {
      if (i == j)
	continue;

      result *= (x - points[i].x()) / (points[j].x() - points[i].x());
    }
    
    return result;
  }
};

#endif // _LagrangePolynomialInterpolation_h








