#ifndef _Point_h
#define _Point_h

// UGLY, but if we don't include <vector>, we don't get <algobase.h>, 
// and we potentially collide if we define our own min and max.
#include <iostream.h>
#include <vector>

#if 0
template <class T> 
T min(const T& a, const T& b)
{
  return ((a < b) ? a : b);
}

template <class T> 
T max(const T& a, const T& b)
{
  return ((a > b) ? a : b);
}
#endif

template <class T>
class Point {
public:
  Point() : 
    px(0), py(0) {}

  Point(const T& x, const T& y) :
    px(x), py(y) {}

  ~Point() {}

  T x() const { return px; }
  T y() const { return py; }

  Point<T>& min(const Point<T>& b) {
    px = ::min(px, b.px);
    py = ::min(py, b.py);
    return *this;
  }

  Point<T>& max(const Point<T>& b) {
    px = ::max(px, b.px);
    py = ::max(py, b.py);
    return *this;
  }

  Point<T> operator+(const Point<T>& b) const {
    return Point<T>(px + b.px, py + b.py);
  }

  Point<T> operator-(const Point<T>& b) const {
    return Point<T>(px - b.px, py - b.py);
  }

  Point<T>& operator-=(const Point<T>& b) {
    px -= b.px;
    py -= b.py;
    return *this;
  }

  Point<T>& operator+=(const Point<T>& b) {
    px += b.px;
    py += b.py;
    return *this;
  }

  Point<T> operator*(const Point<T>& b) const {
    return Point<T>(px * b.px, py * b.py);
  }

  Point<T>& operator*=(const Point<T>& b) {
    px *= b.px;
    py *= b.py;
    return *this;
  }

  Point<T> operator*(const T& b) const {
    return Point<T>(px * b, py * b);
  }

  Point<T>& operator*=(const T& b) {
    px *= b;
    py *= b;
    return *this;
  }

  Point<T> operator/(const Point<T>& b) const {
    return Point<T>(px / b.px, py / b.py);
  }

  Point<T>& operator/=(const Point<T>& b) {
    px /= b.px;
    py /= b.py;
    return *this;
  }

  Point<T> operator/(const T& b) const {
    return Point<T>(px / b, py / b);
  }

  Point<T>& operator/=(T b) {
    px /= b;
    py /= b;
    return *this;
  }

  bool operator==(const Point<T>& b) const {
    return ((px == b.px) && (py == b.py));
  }

  Point<T>& operator=(const Point<T>& b) {
    px = b.px;
    py = b.py;
    return *this;
  }

  void set(const T& x, const T& y) {
    px = x;
    py = y;
  }

private:
  T px, py;

  friend ostream& operator<<(ostream& o, const Point<T>& p);
  friend istream& operator>>(istream& i, Point<T>& p);
};

template <class T>
ostream& operator<<(ostream& o, const Point<T>& p)
{
  o << '(' << p.px << ',' << p.py << ')';

  return o;
}

template <class T>
istream& operator>>(istream& i, Point<T>& p)
{
  int c;

  // Continue searching until we find a '(' or what appears to be a 
  // number.
  do { 
    c = i.get();

    // If we run into what looks like the start of a number, unget it and
    // stop seeking.
    if ((c >= 0) && (isdigit(c) || (c == '.') || (c == '-'))) {
      i.unget();
      break;
    }

  } while ((c >= 0) && (c != '('));

  i >> p.px;

  // Continue searching until we find a ',' or a 'x' or the start of a 
  // number.
  do { 
    c = i.get();

    // If we run into what looks like the start of a number, unget it and
    // stop seeking.
    if ((c >= 0) && (isdigit(c) || (c == '.') || (c == '-'))) {
      i.unget();
      break;
    }

  } while ((c >= 0) && (c != ',') && (c != 'x'));

  i >> p.py;

  // Continue searching until we find a non-blank character, which should
  // be a ')'.  If it's another character, put it back.
  do { 
    c = i.get();

    if ((c >= 0) && !isspace(c)) {
      if (c != ')')
        i.unget();
      break; 
    }
  } while (c >= 0);

  return i;
}


#endif // _Point_h
