#ifndef _Polynomial_h
#define _Polynomial_h

#include <assert.h>
#include <iostream.h>

// A polynomial, with coefficients composed of the type passed as NUMBER.  
// This is intended to support bignum-style classes.  These classes must,
// however, support integer assign and compare operations.  This class is 
// not optimized for extremely large polynomials (degree overflowing an int,
// for example), or for sparse polynomials.

template <class NUMBER>
class Polynomial {
public:
  Polynomial<NUMBER>(const Polynomial& b);
  Polynomial<NUMBER>(int degree =0);

  ~Polynomial();

  Polynomial<NUMBER>& operator=(const Polynomial<NUMBER>& b);

  void setCoefficient(int coeff, const NUMBER& val);
  NUMBER getCoefficient(int coeff) const;

  int getDegree() const;

  // Evaluate f(x).
  NUMBER operator[](NUMBER x) const; 

  Polynomial<NUMBER> operator*(const Polynomial<NUMBER>& b) const;
  Polynomial<NUMBER> operator/(const Polynomial<NUMBER>& b) const;
  Polynomial<NUMBER> operator+(const Polynomial<NUMBER>& b) const;
  Polynomial<NUMBER> operator-(const Polynomial<NUMBER>& b) const;
  Polynomial<NUMBER> operator%(const Polynomial<NUMBER>& b) const;

protected:
  int degree;
  int allocated;
  NUMBER* coefficient;

  // Determine the actual degree of the polynomial
  void reduce();
  
  friend ostream& operator<<(ostream& o, const Polynomial<NUMBER>& poly);
};

#include "Polynomial.i"

#endif // _Polynomial_h
