#include "RNGenerator.h"

RNGenerator::
RNGenerator() : handle(0), isGood(false)
{
}

RNGenerator::~RNGenerator()
{
}

void
RNGenerator::open()
{
  if (isGood)
    close();

  switch (type) {
  case File:
    openFile();
    break;

  case Executable:
    openExecutable();
  break;

  default:
    isGood = false;
    break;
  }
}


bool
RNGenerator::read(unsigned char& ch)
{
  switch (type) {
  case File:
    return readFile(ch);

  case Executable:
    return readExecutable(ch);

  default:
    return false;
  }
}

void
RNGenerator::close()
{
  if ( !isGood )
    return;

  switch (type) {
  case File:
    closeFile();
    break;

  case Executable:
    closeExecutable();
    break;

  default:
    isGood = false;
    break;
  }
}


void 
RNGenerator::setSource(SourceType sourceType, const string& sourceName)
{
  type = sourceType;
  name = sourceName;
}

// Copy an RNGenerator.  It has the same initialization information, but not
// state.  In other words, once copied, you'll have to run open() to get it
// to work properly.
RNGenerator&
RNGenerator::operator=(const RNGenerator& from)
{
  type = from.type;
  name = from.name;
  handle = 0;

  isGood = false;

  return *this;
}

void 
RNGenerator::openFile()
{
  handle = fopen(name.c_str(), "r");

  if (handle)
    isGood = true;
}

bool
RNGenerator::readFile(unsigned char& ch)
{
  if ( !handle )
    return false;

  int result = fgetc(handle);

  if (result < 0) {
    isGood = false;
    return false;
  } else {
    ch = (unsigned char) result;
    return true;
  }
}

void 
RNGenerator::closeFile()
{
  if (isGood && handle)
    fclose(handle);

  isGood = false;
}

void 
RNGenerator::openExecutable()
{
  handle = popen(name.c_str(), "r");

  if (handle)
    isGood = true;
}

bool
RNGenerator::readExecutable(unsigned char& ch)
{
  return readFile(ch);
}

void
RNGenerator::closeExecutable()
{
  if (handle)
    pclose(handle);

  isGood = false;
}
