#ifndef _RNGenerator_h
#define _RNGenerator_h

#include <string>
using namespace std;		// some C++ compilers are fussy

extern "C" {
  #include <stdio.h>
};

// Attempts to encapsulate random number generators from several different
// sources.  You can set the source to be a file, executable, built-in hash,
// rand48() output on a rotating-but-probably-not-hard-to-guess seed, or
// random().  

class RNGenerator {
public:
  RNGenerator();
  ~RNGenerator();
  
  void open();
  bool read(unsigned char& ch);
  void close();

  enum SourceType {
    None,
    File,
    Executable
  };

  void setSource(SourceType type, const string& name);

  RNGenerator& operator=(const RNGenerator& from);

private:
  void openFile();
  bool readFile(unsigned char& ch);
  void closeFile();

  void openExecutable();
  bool readExecutable(unsigned char& ch);
  void closeExecutable();

  SourceType type;
  string name;
  FILE* handle;

  bool isGood;
};

#endif // _RNGenerator_h









