#include <strstream.h>
//#include <fstream.h>
#include <stdlib.h>
#include "SecretShare.h"
#include "FIOstream.h"

#include "LagrangePolynomialInterpolation.h"

SecretShare::SecretShare(int keys, int keysNeeded, bool stego) :
  keys(keys), keysNeeded(keysNeeded), stego(stego), rand(), prefix("key.")
{
}

SecretShare::~SecretShare()
{
}

void 
SecretShare::encode(istream& stream)
{
  // rand.setSource(RNGenerator::Executable, "/usr/bin/yes");
  rand.setSource(RNGenerator::File, "/dev/urandom");
  
  rand.open();

  //ofiles = new (FIOstream*)[keys];
  // Sun CC objects to above -- should it?
  ofiles = new FIOstreamp[keys];
  for (int i = 0; i < keys; i++) {
    ostrstream name;
    name << prefix.c_str() << i << ends;
    ofiles[i] = new FIOstream (name.str());
  }

  char ch;

  while (stream) {
    stream.get(ch);

    if (stream)
      encode(ch);
  }

  if (stego)
    for (int i = 0; i < keys; i++) {
      appendNoise(ofiles[i]);
    }

  for (int i=0; i<keys; i++)
    delete ofiles[i];
  delete[] ofiles;

  rand.close();
}

void 
SecretShare::encode(unsigned char ch)
{
  Polynomial<GF256> p(keysNeeded - 1);

  p.setCoefficient(0, GF256(ch));

  for (int i = 1; i <= keysNeeded - 1; i++) {
    unsigned char ch;

    assert(rand.read(ch));

    p.setCoefficient(i, GF256(ch));
  }

  createKeys(p);

  // cout << "Encoder = " << p << endl;
}

// Currently, every key file is opened, to append bytes, one at a time.
// Ideally, we would use block-based buffers and commit them every N bytes.
// I'll do this Real Soon Now. 
void 
SecretShare::createKeys(const Polynomial<GF256>& p)
{
  int* usedKeys = new int[keys];

  for (int i = 0; i < keys; i++) {
    unsigned char ch;
    GF256 x, y;

    do {
      assert(rand.read(ch));

      x = GF256(ch);

      // If we happen to pick 0 for x, it would encode the secret literally.
      // This is obviously a bad thing.  However, if we fail to include a 
      // 0, it's a red flag that this isn't random data, because every other
      // byte is never 0.  So, we put the 0 there, and then throw in a second
      // random byte.  Then, in decoding, if we spot a 0, we skip that pair.
      if ( x == 0 && stego ) {
	cout << "BOGUS CHAR PUSHED ON KEY " << i << endl;
	ofiles[i]->write(0);
	assert(rand.read(ch));
	ofiles[i]->write(ch);
      } else { 
	y = p[x];
      }

    } while ((x == 0) || keyUsed(usedKeys, i - 1, x));

    usedKeys[i] = x;

    ofiles[i]->write(x);
    ofiles[i]->write(y);
  }

  delete[] usedKeys;
}

// Takes an output stream, and (keys) input streams, and does shared secret
// extraction.
bool
//SecretShare::decode(ostream& out, ifstream* inputs)
SecretShare::decode(ostream& out, FIOstream** inputs)
{
  Point<GF256>* p = new Point<GF256>[keysNeeded];
  GF256 x, y;
  bool done = false;
  bool isGood = true;
  ostrstream encapStream;
  unsigned char ch;

  do {
    for (int i = 0; i < keysNeeded; i++) {
      //inputs[i].get(ch);
      ch = inputs[i]->read();
      x = GF256((int) ch);

      // If we encounter a case where x is 0, this is a bogus pair, simply
      // included to maintain stream entropy.
      while ( x == 0 && !inputs[0]->eof()) {
	// Get the bogus y and a new x.
	//inputs[i].get(ch);
	//inputs[i].get(ch);
	ch = inputs[i]->read();
	ch = inputs[i]->read();
	x = GF256((int) ch);
      }

      //inputs[i].get(ch);
      ch = inputs[i]->read();
      y = GF256((int) ch);

      //if (!inputs[i])
      if (inputs[i]->weirdTest())// what does this test do?  EOF?
	done = true;
      else {
	p[i].set(x, y);
      }
    }

    if (!done) {
      if (!decode(out, p)) {
	isGood = false;
	break;
      }
    }

    // cout << "Decoded block" << endl;

  } while (!done);

  delete[] p;

  if (!isGood)
    return false;
  else
    return true;
}

// A 20-33% speed increase (especially for large block sizes) was obtained 
// by substituting the fit(0) interpolation for fit(), so we're only finding
// the f(0), instead of finding the interpolating polynomial and then 
// finding f(0) of it.
bool
SecretShare::decode(ostream& out, const Point<GF256>* inputs)
{
  LagrangePolynomialInterpolation<GF256> LPI;

  for (int i = 0; i < keysNeeded; i++)
    LPI.addPoint(inputs[i]);

  if (!LPI.uniquePoints())
    return false;

  // This really should be LPI.fit(0), but for some reason, that doesn't
  // work (yet?).
  GF256 fit = LPI.fit()[0];

  out.put((unsigned char) (int) fit);

  return true;
}

void 
SecretShare::setPrefix(const string& newPrefix)
{
  prefix = newPrefix;
}

string
SecretShare::getPrefix() const
{
  return prefix;
}

void
SecretShare::setRNG(const RNGenerator& rn)
{
  rand.close();
  rand = rn;
}

/*
void
SecretShare::createBuffers()
{
  charsPushed = new int[keys];
  buffer = new char*[keys];

  for (int i = 0; i < keys; i++) {
    charsPushed[i] = 0;
    buffer[i] = new char[bufferSize];
  }
}

void 
SecretShare::flush()
{
  for (int i = 0; i < keys; i++) {
    if ( !charsPushed[i] )
      continue;

    ofstream out;
  
    outfile(out, i, true);

    out.write(buffer[i], charsPushed[i]);

    out.close();

    charsPushed[i] = 0;
  }
}

void
SecretShare::deleteBuffers()
{
  for (int i = 0; i < keys; i++)
    delete[] buffer[i];
  
  delete[] charsPushed;
  delete[] buffer;
}
*/

// Add a variable amount of noise from the RNG.  This amount should be
// configurable, either as a variable range or a variable percentage of the 
// file size.  But for now, we'll just add 0-2k of noise.  The point here 
// is to make it unclear which keys go together.  If keys are stored in a 
// noise-rich environment, it's difficult to say "ok, we now have 2 of the
// keys".  Maybe you do, maybe you don't.  But if 2 people have files with
// random noise, each exactly 3888 bytes, it's a little more suspicious.
// 
// If this is not an issue, simply set the variable amount to 0.  Well, 
// when that feature's implemented, anyway.
#if 0
void
SecretShare::appendNoise(ofstream& /* out */)
{    
}
#endif
void SecretShare::appendNoise (FIOstream *f)
{
}
/*
void 
SecretShare::outfile(ofstream& file, int i, bool append)
{
  ostrstream name;

  name << prefix.c_str() << i << ends;
  
  if (append)
    file.open(name.str(), ios::app);
  else
    file.open(name.str());

  name.rdbuf()->freeze(0);
}

void
SecretShare::infile(ifstream& file, int i)
{
  ostrstream name;

  name << "encoded." << i << ends;
  
  file.open(name.str(), ios::app);
  
  name.rdbuf()->freeze(0);
}
*/

bool 
SecretShare::keyUsed(int* usedKeys, int index, int x) const
{
  for (int i = 0; i <= index; i++) {
    if (usedKeys[i] == x) {
      return true;
    }
  }

  return false;
}

